// -*- C++ -*-
/*!
 * @file  ImageLoader.cpp
 * @brief image acquisition component
 * @date $Date$
 *
 * @author 株式会社 東日本計算センター
 *
 * $Id$
 */

#include "ImageLoader.h"
#include <shlwapi.h>

#pragma comment(lib, "ShLwApi.Lib")

// add to code
#define DEBUG_SW

// Module specification
// <rtc-template block="module_spec">
static const char* imageloader_spec[] =
  {
    "implementation_id", "ImageLoader",
    "type_name",         "ImageLoader",
    "description",       "image acquisition component",
    "version",           "1.0",
    "vendor",            "EAC",
    "category",          "image-acquisition",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.Imloader_file1path", "C:/ImageFile_Dir/ImageLoaderFile.bmp",
    "conf.default.Imloader_file2path", "C:/ImageFile_Dir/ImageLoaderFile.bmp",
    "conf.default.Imloader_file1switch", "0",
    "conf.default.Imloader_file2switch", "0",
    // Widget
    "conf.__widget__.Imloader_file1path", "text",
    "conf.__widget__.Imloader_file2path", "text",
    "conf.__widget__.Imloader_file1switch", "radio",
    "conf.__widget__.Imloader_file2switch", "radio",
    // Constraints
    "conf.__constraints__.Imloader_file1switch", "(0,1)",
    "conf.__constraints__.Imloader_file2switch", "(0,1)",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
ImageLoader::ImageLoader(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_in1_pathIn("ImagePathIn_1", m_in1_path),
    m_in2_pathIn("ImagePathIn_2", m_in2_path),
    m_out1_imageOut("ImageDataOut_1", m_out1_image),
    m_out2_imageOut("ImageDataOut_2", m_out2_image),
    m_ImageLoaderControlServPort("ImageLoaderControlServ"),
    m_ImageLoaderSystemContServPort("ImageLoaderSystemContServ")

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
ImageLoader::~ImageLoader()
{
}


/*!
 * Initialization of ImageLoader RTC
 */
RTC::ReturnCode_t ImageLoader::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("ImagePathIn_1", m_in1_pathIn);
  addInPort("ImagePathIn_2", m_in2_pathIn);
  
  // Set OutPort buffer
  addOutPort("ImageDataOut_1", m_out1_imageOut);
  addOutPort("ImageDataOut_2", m_out2_imageOut);
  
  // Set service provider to Ports
  m_ImageLoaderControlServPort.registerProvider("ImageLoaderTrigger", "ImageLoaderControlServ::ImageLoaderTrigger", m_ImageLoaderTriggerProv);
  m_ImageLoaderSystemContServPort.registerProvider("ImageLoaderReset", "ImageLoaderSystemContServ::ImageLoaderReset", m_ImageLoaderResetProv);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  addPort(m_ImageLoaderControlServPort);
  addPort(m_ImageLoaderSystemContServPort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("Imloader_file1path", m_file1path_str, "C:/ImageFile_Dir/ImageLoaderFile.bmp");
  bindParameter("Imloader_file2path", m_file2path_str, "C:/ImageFile_Dir/ImageLoaderFile.bmp");
  bindParameter("Imloader_file1switch", m_imld_file1sw, "0");
  bindParameter("Imloader_file2switch", m_imld_file2sw, "0");
  // </rtc-template>

  // add to code
  std::cout << "onInitialize execution" << std::endl;
  m_stFilePath.path_fname1 = "C:/dummy_dummy1";
  m_stFilePath.path_fname2 = "C:/dummy_dummy2";
  m_stFilePath.validfname1 = IMLD_FILE_NAME_INVALID;
  m_stFilePath.validfname2 = IMLD_FILE_NAME_INVALID;
  m_in_path_fname = "C:";
  
  uWErrorCounter = IMLD_ERROR_COUNT_INIT;
  
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ImageLoader::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ImageLoader::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ImageLoader::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * Activate the ImageLoader RTC
 */

RTC::ReturnCode_t ImageLoader::onActivated(RTC::UniqueId ec_id)
{
  // add to code
  std::cout << "onActivated execution" << std::endl;
  m_nuBImldInPortNum = IMLD_IN_INIT;
  m_stFilePath.path_fname1 = "C:/dummy_dummy1";
  m_stFilePath.path_fname2 = "C:/dummy_dummy2";
  m_stFilePath.validfname1 = IMLD_FILE_NAME_INVALID;
  m_stFilePath.validfname2 = IMLD_FILE_NAME_INVALID;
  m_in_path_fname = "C:";

  return RTC::RTC_OK;
}

/*!
 * Deactivate the ImageLoader RTC
 */

RTC::ReturnCode_t ImageLoader::onDeactivated(RTC::UniqueId ec_id)
{
  // add to code
  std::cout << "onDeactivated execution" << std::endl;
  m_nuBImldInPortNum = IMLD_IN_INIT;
  m_stFilePath.path_fname1 = "C:/dummy_dummy1";
  m_stFilePath.path_fname2 = "C:/dummy_dummy2";
  m_stFilePath.validfname1 = IMLD_FILE_NAME_INVALID;
  m_stFilePath.validfname2 = IMLD_FILE_NAME_INVALID;

  m_stFilePath_shadow.validfname1 = IMLD_FILE_NAME_INVALID;
  m_stFilePath_shadow.validfname2 = IMLD_FILE_NAME_INVALID;

  m_in_path_fname = "C:";

  return RTC::RTC_OK;
}

/*!
 * Periodic task of ImageLoader RTC. This task read the image
 * file.
 */

RTC::ReturnCode_t ImageLoader::onExecute(RTC::UniqueId ec_id)
{
  // add to code
  uWord_Type auWResult = IMLD_EXE_INIT;

  nvdSetImageLoaderFilePath();           //output m_stFilePath
  auWResult = nuWImageLoaderExecute();   //input  m_stFilePath
  if( auWResult != IMLD_EXE_SUCCESS )
  {
    return RTC::RTC_ERROR;
  }

  return RTC::RTC_OK;
}

/*!
 * Aborting of the ImageLoader RTC
 */

RTC::ReturnCode_t ImageLoader::onAborting(RTC::UniqueId ec_id)
{
  // add to code
  std::cout << "onAborting execution" << std::endl;
  m_stFilePath.path_fname1 = "C:/dummy_dummy1";
  m_stFilePath.path_fname2 = "C:/dummy_dummy2";
  m_stFilePath.validfname1 = IMLD_FILE_NAME_INVALID;
  m_stFilePath.validfname2 = IMLD_FILE_NAME_INVALID;

  m_stFilePath_shadow.validfname1 = IMLD_FILE_NAME_INVALID;
  m_stFilePath_shadow.validfname2 = IMLD_FILE_NAME_INVALID;

  m_ImageLoaderTriggerProv.ReqImageLoaderTrigger( IMLD_TRIG_STOP );
  uWErrorCounter = IMLD_ERROR_COUNT_MAX;

  return RTC::RTC_OK;
}

/*!
 * Errored of the ImageLoader RTC
 */

RTC::ReturnCode_t ImageLoader::onError(RTC::UniqueId ec_id)
{
  // add to code
  if( uWErrorCounter != IMLD_ERROR_COUNT_INIT )
  {
    if( uWErrorCounter >= IMLD_ERROR_COUNT_MIN )
    {
      uWErrorCounter = uWErrorCounter - 1;
      if( (uWErrorCounter == IMLD_ERROR_COUNT_MIN) || (uWErrorCounter == IMLD_ERROR_COUNT_RESTART) )
      {
        std::cout << "onError execution" << std::endl;
        uWErrorCounter = IMLD_ERROR_COUNT_RESTART;
      }
    }
  }
  else
  {
    uWErrorCounter = IMLD_ERROR_COUNT_RESTART;
  }

  uWord_Type uWReqReset = m_ImageLoaderResetProv.GetImageLoaderReset();

  if (uWReqReset == RESET_TRIG_ON)
  {
	  get_context(ec_id)->reset_component( getObjRef() );

	  //Initializes the reset request
	  m_ImageLoaderResetProv.ReqImageLoaderReset( RESET_TRIG_OFF );

  }

  return RTC::RTC_OK;
}

/*!
 * Reset initialization of ImageLoader RTC
 */

RTC::ReturnCode_t ImageLoader::onReset(RTC::UniqueId ec_id)
{
  // add to code
  std::cout << "onReset execution" << std::endl;
  uWErrorCounter = IMLD_ERROR_COUNT_INIT;

#if 0 //Go to activate
	get_context(ec_id)->activate_component( getObjRef() );
#endif
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ImageLoader::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ImageLoader::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


/***************************************************************************************************
 *  Function Name   ：nvdSetImageLoaderFilePath
 *  Description     ：This function sets the file full path information to the member variables.
 *  Argument        ：nothing special
 *  Return          ：nothing special
 **************************************************************************************************/
void ImageLoader::nvdSetImageLoaderFilePath(void)
{
  uWord_Type auWImldTrigger = m_ImageLoaderTriggerProv.GetImageLoaderTrigger();

  //assign a beginning value to a variable
  m_stFilePath.validfname1 = IMLD_FILE_NAME_INVALID;
  m_stFilePath.validfname2 = IMLD_FILE_NAME_INVALID;

  nvdJudgeImageLoaderFilePath( auWImldTrigger, IMLD_IN_PORT1 );
  nvdJudgeImageLoaderFilePath( auWImldTrigger, IMLD_IN_PORT2 );

}


/***************************************************************************************************
 *  Function Name   ：nvdJudgeImageLoaderFilePath
 *  Description     ：This function to confirmed the input of the new full path information.
 *                  ：And sets a member variables of the image-loader.
 *  Argument        ：auWImldTrigger, auBImldInPortNum
 *  Return          ：nothing special
 **************************************************************************************************/
void ImageLoader::nvdJudgeImageLoaderFilePath( uWord_Type auWImldTrigger, uByte_Type auBImldInPortNum )
{
  if( auWImldTrigger == IMLD_TRIG_START )
  {
    if( nuWGetFileSwConfig( auBImldInPortNum ) == IMLD_FILE_LOAD_DISABLE )
    {
      if( nuWJudgeNewPathIn( auBImldInPortNum ) == IMLD_PATH_IN_VALID )
      {
        nvdSetFilePath( m_in_path_fname, auBImldInPortNum, FILE_PATH_OPT_ON );
      }
      else
      {
        if( nuWJudgeShadowPathIn( auBImldInPortNum ) == IMLD_FILE_NAME_VALID )
        {
          if( auBImldInPortNum == IMLD_IN_PORT1 )
          {
            nvdSetFilePath( m_stFilePath_shadow.path_fname1, auBImldInPortNum, FILE_PATH_OPT_ON);
          }
          else if( auBImldInPortNum == IMLD_IN_PORT2 )
          {
            nvdSetFilePath( m_stFilePath_shadow.path_fname2, auBImldInPortNum, FILE_PATH_OPT_ON);
          }
          else
          {
          }
        }
      }
    }
    else
    {
      if( nuWCompareFilePath( auBImldInPortNum ) == IMLD_FILE_PATH_UNMATCH )
      {
        if( auBImldInPortNum == IMLD_IN_PORT1 )
        {
          nvdSetFilePath( m_file1path_str, auBImldInPortNum, FILE_PATH_OPT_OFF );
        }
        else if( auBImldInPortNum == IMLD_IN_PORT2 )
        {
          nvdSetFilePath( m_file2path_str, auBImldInPortNum, FILE_PATH_OPT_OFF );
        }
        else
        {
        }
      }

      if( nuWJudgeNewPathIn( auBImldInPortNum ) == IMLD_PATH_IN_VALID )
      {
        nvdSetFilePath( m_in_path_fname, auBImldInPortNum, FILE_PATH_OPT_OFF );
      }
    }
  }
  else
  {
    if( nuWJudgeNewPathIn( auBImldInPortNum ) == IMLD_PATH_IN_VALID )
    {
      nvdSetFilePathShadow( m_in_path_fname, auBImldInPortNum );
    }
    m_stFilePath.path_fname1 = "C:/dummy_dummy1";
    m_stFilePath.path_fname2 = "C:/dummy_dummy2";
  }
}


/***************************************************************************************************
 *  Function Name   ：nuWImageLoaderExecute
 *  Description     ：1.This function to confirmed the extension of the image file.
 *                  ：2.This function will confirmed the read of image file if it is correct.
 *                  ：If these results are correct, it executed the analysis of the image file.
 *  Argument        ：nothing special
 *  Return          ：analysis results of the image file
 **************************************************************************************************/
uWord_Type ImageLoader::nuWImageLoaderExecute( void )
{
  uWord_Type auWResult = IMLD_EXE_SUCCESS;

  cv::Mat dsp_image1;
  cv::Mat dsp_image2;

  //Confirmation of input1 ------------------------------------------------------------
  if(m_stFilePath.validfname1 == IMLD_FILE_NAME_VALID)
  {
    m_nuBImldInPortNum  = IMLD_IN_PORT1;
    auWResult = nuWJudgeFileFormat( m_stFilePath.path_fname1 );
    if( auWResult == FILE_FORMAT_FAILURE )
    {
      std::cout << UNSUPPORED_FORMAT << std::endl;
      auWResult = IMLD_EXE_SUCCESS;
      return auWResult;
    }

    dsp_image1 = cv::imread(m_stFilePath.path_fname1, cv::IMREAD_UNCHANGED);
    if (dsp_image1.empty())
    {
      std::cout << EMPTY_EROR << std::endl;
      auWResult = IMLD_EXE_FAILURE;
      return auWResult;
    }

    auWResult = nuWAnalysisImageLoader( m_stFilePath.path_fname1, IMLD_IN_PORT1 );
    if( auWResult == IMLD_EXE_SUCCESS )
    {
      std::cerr << "port1 success" << std::endl;
    }
    else
    {
      std::cerr << "port1 failure" << std::endl;
    }
  }

  //Confirmation of input2 ------------------------------------------------------------
  auWResult = IMLD_EXE_SUCCESS;

  if(m_stFilePath.validfname2 == IMLD_FILE_NAME_VALID)
  {
    m_nuBImldInPortNum  = IMLD_IN_PORT2;
    auWResult = nuWJudgeFileFormat( m_stFilePath.path_fname2 );
    if( auWResult == FILE_FORMAT_FAILURE )
    {
      std::cout << UNSUPPORED_FORMAT << std::endl;
      auWResult = IMLD_EXE_SUCCESS;
      return auWResult;
    }

    dsp_image2 = cv::imread(m_stFilePath.path_fname2, cv::IMREAD_UNCHANGED);
    if (dsp_image2.empty())
    {
      std::cout << EMPTY_EROR << std::endl;
      auWResult = IMLD_EXE_FAILURE;
      return auWResult;
    }

    auWResult = nuWAnalysisImageLoader( m_stFilePath.path_fname2, IMLD_IN_PORT2 );
    if( auWResult == IMLD_EXE_SUCCESS )
    {
      std::cerr << "port2 success" << std::endl;
    }
    else
    {
      auWResult = IMLD_EXE_FAILURE;
      std::cerr << "port2 failure" << std::endl;
    }
  }

  return auWResult;
}


/***************************************************************************************************
 *  Function Name   ：nuWAnalysisImageLoader
 *  Description     ：After analyzing the image file is converted into the CameraImage type data.
 *                  ：If the result of the analysis is correct, it outputs the CameraImage type of data.
 *  Argument        ：nothing special
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type ImageLoader::nuWAnalysisImageLoader(std::string apath_fname, uByte_Type auBImldInPortNum)
{
  uWord_Type auWReturn           = IMLD_EXE_FAILURE;
  uWord_Type auWImldSetResult    = IMLD_SET_FAILURE;

  nvdSetImage_fDiv( 0, auBImldInPortNum );

  m_dsp_image = cv::imread(apath_fname, cv::IMREAD_UNCHANGED);
  nvdSetImage_Width( &m_dsp_image, auBImldInPortNum );
  nvdSetImage_Height( &m_dsp_image, auBImldInPortNum );

  if(m_dsp_image.channels() == 1)
  {
    nvdSetImage_Bpp( IMLD_BPP_8, auBImldInPortNum );
  }
  else if(m_dsp_image.channels() == 3)
  {
    nvdSetImage_Bpp( IMLD_BPP_24, auBImldInPortNum );
  }
  else
  {
    nvdSetImage_Bpp( IMLD_BPP_INIT, auBImldInPortNum );
#ifdef DEBUG_SW
    std::cout << UNSUPPORED_BPP << std::endl;
#endif
    return auWReturn;
  }

#ifdef DEBUG_SW
  std::cout << "cols: " << m_dsp_image.cols <<std::endl;
  std::cout << "rows: " << m_dsp_image.rows <<std::endl;
  std::cout << "channels: " << m_dsp_image.channels() << std::endl;
#endif

  nvdSetImage_Length( &m_current_image, auBImldInPortNum );

  if( auBImldInPortNum == IMLD_IN_PORT1 )
  {
    auWImldSetResult = nuWSetImage_Pixels1( m_dsp_image, &m_current_image );
  }
  else if( auBImldInPortNum == IMLD_IN_PORT2 )
  {
    auWImldSetResult = nuWSetImage_Pixels2( m_dsp_image, &m_current_image );
  }
  else
  {
    auWImldSetResult = IMLD_SET_FAILURE;
  }

  if( auWImldSetResult == IMLD_SET_SUCCESS)
  {
    auWReturn = IMLD_EXE_SUCCESS;
  }
  else
  {
    auWReturn = IMLD_EXE_FAILURE;
  }

  return auWReturn;

}

/***************************************************************************************************
 *  Function Name   ：nuWJudgeFileExtension
 *  Description     ：This function to confirmed the extension of the image file.
 *  Argument        ：apath_fname, aStrFileFormat
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type ImageLoader::nuWJudgeFileExtension(std::string apath_fname, std::string *aStrFileFormat)
{
  uWord_Type auWReturn = FILE_FORMAT_FAILURE;
  std::string aStrExtension;

  std::tr2::sys::path path(apath_fname);
  aStrExtension = path.extension();
#ifdef DEBUG_SW
  std::cout << "extension:" << path.extension() << std::endl;
#endif

  //Lowercase conversion of extension.
  std::transform(aStrExtension.begin(), aStrExtension.end(), aStrExtension.begin(), tolower);
  
  if( (aStrExtension.compare(".bmp") == 0) || (aStrExtension.compare(".dib") == 0) )
    {
      *aStrFileFormat = "bitmap";
      auWReturn = FILE_FORMAT_SUCCESS;
    }
    else if( (aStrExtension.compare(".jpeg") == 0) || (aStrExtension.compare(".jpg") == 0) || (aStrExtension.compare(".jpe") == 0) )
    {
      *aStrFileFormat = "jpeg";
      auWReturn = FILE_FORMAT_SUCCESS;
    }
    else if( aStrExtension.compare(".png") == 0 )
    {
      *aStrFileFormat = "png";
      auWReturn = FILE_FORMAT_SUCCESS;
    }
    else if( (aStrExtension.compare(".tiff") == 0) || (aStrExtension.compare(".tif") == 0) )
    {
      *aStrFileFormat = "tiff";
      auWReturn = FILE_FORMAT_SUCCESS;
    }
    else
    {
      *aStrFileFormat = "unsupported";
    }

  return auWReturn;

}


/***************************************************************************************************
 *  Function Name   ：nuWJudgeFileFormat
 *  Description     ：This function executed the formatting confirmation of the image file.
 *  Argument        ：apath_fname
 *  Return          ：information of normal end or abnormal end.
**************************************************************************************************/
uWord_Type ImageLoader::nuWJudgeFileFormat(std::string apath_fname)
{
  Char_Type   aCMarker[FORMAT_MARKER_MAX]  = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
  uByte_Type  auBMarker[FORMAT_MARKER_MAX] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
  uWord_Type  auWReturn      = FILE_FORMAT_FAILURE;
  std::string aStrFileFormat = "unsupported";

  nvdSetImage_Format( "unsupported", m_nuBImldInPortNum );

  auWReturn = nuWJudgeFileExtension( apath_fname, &aStrFileFormat );

  if( auWReturn != FILE_FORMAT_SUCCESS )
  {
#ifdef DEBUG_SW
    std::cout << UNSUPPORED_EXTENSION << std::endl;
#endif
    auWReturn = FILE_FORMAT_FAILURE;
    return auWReturn;
  }
  else
  {
    auWReturn = FILE_FORMAT_FAILURE;

    std::ifstream aifs_fmt(apath_fname);
    if(aifs_fmt.fail())
    {
      std::cout << READING_ERROR << std::endl;
    }
    else
    {
      aifs_fmt.getline(aCMarker, FORMAT_MARKER_MAX - 1);
      std::istringstream is_numeric(aCMarker);
      is_numeric >> auBMarker;

      if( aStrFileFormat.compare("bitmap") == 0 )
      {
        auWReturn = nuWJudgeFileFormatBitmap( auBMarker[0], auBMarker[1] );
      }
      else if( aStrFileFormat.compare("jpeg") == 0 )
      {
        auWReturn = nuWJudgeFileFormatJpeg( auBMarker[0], auBMarker[1], auBMarker[2] );
      }
      else if( aStrFileFormat.compare("png") == 0 )
      {
        auWReturn = nuWJudgeFileFormatPng( auBMarker[0], auBMarker[1], auBMarker[2], auBMarker[3] );
      }
      else if( aStrFileFormat.compare("tiff") == 0 )
      {
        auWReturn = nuWJudgeFileFormatTiff( auBMarker[0], auBMarker[1], auBMarker[2] );
      }
      else
      {
        std::cout << UNSUPPORED_FORMAT << std::endl;
      }
    }
  }

  return auWReturn;

}


/***************************************************************************************************
 *  Function Name   ：nuWJudgeFileFormatBitmap
 *  Description     ：This function executed the format confirmation of Bitmap format.
 *  Argument        ：marker information of the image data.
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type ImageLoader::nuWJudgeFileFormatBitmap( uByte_Type auBMarker_0, uByte_Type auBMarker_1 )
{
  uWord_Type auWReturn = FILE_FORMAT_FAILURE;

  nvdSetImage_Format( "unsupported", m_nuBImldInPortNum );

  if( (auBMarker_0 == FORMAT_BITMAP_B) && (auBMarker_1 == FORMAT_BITMAP_M) )
  {
    nvdSetImage_Format( "bitmap", m_nuBImldInPortNum );
    auWReturn = FILE_FORMAT_SUCCESS;
    std::cout << "format: bitmap" << std::endl;
  }
  else
  {
#ifdef DEBUG_SW
    std::cout << UNSUPPORED_BITMAP << std::endl;
#endif
  }

  return auWReturn;
}


/***************************************************************************************************
 *  Function Name   ：nuWJudgeFileFormatJpeg
 *  Description     ：This function executed the format confirmation of Jpeg format.
 *  Argument        ：marker information of the image data.
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type ImageLoader::nuWJudgeFileFormatJpeg( uByte_Type auBMarker_0, uByte_Type auBMarker_1, uByte_Type auBMarker_2 )
{
  uWord_Type auWReturn = FILE_FORMAT_FAILURE;

  nvdSetImage_Format( "unsupported", m_nuBImldInPortNum );

  if( (auBMarker_0 == FORMAT_JPEG_1ST) && (auBMarker_1 == FORMAT_JPEG_2ND) && (auBMarker_2 == FORMAT_JPEG_3RD) )
  {
    nvdSetImage_Format( "jpeg", m_nuBImldInPortNum );
    auWReturn = FILE_FORMAT_SUCCESS;
    std::cout << "format: jpeg" << std::endl;
  }
  else
  {
#ifdef DEBUG_SW
    std::cout << UNSUPPORED_JPEG << std::endl;
#endif
  }

  return auWReturn;
}


/***************************************************************************************************
 *  Function Name   ：nuWJudgeFileFormatPng
 *  Description     ：This function executed the format confirmation of Png format.
 *  Argument        ：marker information of the image data.
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type ImageLoader::nuWJudgeFileFormatPng( uByte_Type auBMarker_0, uByte_Type auBMarker_1, uByte_Type auBMarker_2, uByte_Type auBMarker_3 )
{
  uWord_Type auWReturn = FILE_FORMAT_FAILURE;

  nvdSetImage_Format( "unsupported", m_nuBImldInPortNum );

  if( (auBMarker_0 == FORMAT_PNG_1ST) && (auBMarker_1 == FORMAT_PNG_2ND) && (auBMarker_2 == FORMAT_PNG_3RD) && (auBMarker_3 == FORMAT_PNG_4TH) )
  {
    nvdSetImage_Format( "png", m_nuBImldInPortNum );
    auWReturn = FILE_FORMAT_SUCCESS;
    std::cout << "format: png" << std::endl;
   }
   else
   {
#ifdef DEBUG_SW
    std::cout << UNSUPPORED_PNG << std::endl;
#endif
   }

  return auWReturn;
}


/***************************************************************************************************
 *  Function Name   ：nuWJudgeFileFormatTiff
 *  Description     ：This function executed the format confirmation of Tiff format.
 *  Argument        ：marker information of the image data.
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type ImageLoader::nuWJudgeFileFormatTiff( uByte_Type auBMarker_0, uByte_Type auBMarker_1, uByte_Type auBMarker_2 )
{
  uWord_Type auWReturn = FILE_FORMAT_FAILURE;

  nvdSetImage_Format( "unsupported", m_nuBImldInPortNum );

  if( ( (auBMarker_0 == FORMAT_TIF_1ST_TYP1) && (auBMarker_1 == FORMAT_TIF_2ND_TYP1) && (auBMarker_2 == FORMAT_TIF_3RD_TYP1) )
  ||  ( (auBMarker_0 == FORMAT_TIF_1ST_TYP2) && (auBMarker_1 == FORMAT_TIF_2ND_TYP2) && (auBMarker_2 == FORMAT_TIF_3RD_TYP2) ) )
  {
    nvdSetImage_Format( "tiff", m_nuBImldInPortNum );
    auWReturn = FILE_FORMAT_SUCCESS;
    std::cout << "format: tiff" << std::endl;
  }
  else
  {
#ifdef DEBUG_SW
    std::cout << UNSUPPORED_TIFF << std::endl;
#endif
  }

  return auWReturn;
}


/***************************************************************************************************
 *  Function Name   ：nvdSetImage_fDiv
 *  Description     ：This function sets the fDiv data to CameraImage type data.
 *  Argument        ：fDiv data of image
 *  Return          ：nothing special
 **************************************************************************************************/
void ImageLoader::nvdSetImage_fDiv( Double_Type aDb_fDiv, uByte_Type auBPortInNum )
{
  if( auBPortInNum < IMLD_IN_PORT_MAX )
  {
    m_current_image.fDiv = aDb_fDiv;

    if( auBPortInNum == IMLD_IN_PORT1 )
    {
      m_out1_image.fDiv = aDb_fDiv;
    }
    else if( auBPortInNum == IMLD_IN_PORT2 )
    {
      m_out2_image.fDiv = aDb_fDiv;
    }
    else
    {
      //IMLD_SET_FAILURE;
    }
  }
}


/***************************************************************************************************
 *  Function Name   ：nvdSetImage_Format
 *  Description     ：This function sets the format data to CameraImage type data.
 *  Argument        ：format data of image
 *  Return          ：nothing special
 **************************************************************************************************/
void ImageLoader::nvdSetImage_Format( std::string aImageFormat, uByte_Type auBPortInNum )
{
  uWord_Type auWFormatSetState = IMLD_SET_SUCCESS;

  if(aImageFormat.compare("bitmap") == 0)
  {
    m_current_image.format = "bitmap";
  }
  else if(aImageFormat.compare("jpeg") == 0)
  {
    m_current_image.format = "jpeg";
  }
  else if(aImageFormat.compare("png") == 0)
  {
    m_current_image.format = "png";
  }
  else if(aImageFormat.compare("tiff") == 0)
  {
    m_current_image.format = "tiff";
  }
  else if(aImageFormat.compare("unsupported") == 0)
  {
    m_current_image.format = "unsupported";
  }
  else
  {
    m_current_image.format = "unsupported";
    auWFormatSetState = IMLD_SET_FAILURE;
  }

  if( (auBPortInNum < IMLD_IN_PORT_MAX) && (auWFormatSetState == IMLD_SET_SUCCESS) )
  {
    if( auBPortInNum == IMLD_IN_PORT1 )
    {
      m_out1_image.format = m_current_image.format;
    }
    else if( auBPortInNum == IMLD_IN_PORT2 )
    {
      m_out2_image.format = m_current_image.format;
    }
    else
    {
      m_current_image.format = "unsupported";
      auWFormatSetState = IMLD_SET_FAILURE;
    }
  }
  else
  {
    m_current_image.format = "unsupported";
    auWFormatSetState = IMLD_SET_FAILURE;
  }
}

/***************************************************************************************************
 *  Function Name   ：nvdSetImage_Width
 *  Description     ：This function sets the width data to CameraImage type data.
 *  Argument        ：width data of image
 *  Return          ：nothing special
 **************************************************************************************************/
void ImageLoader::nvdSetImage_Width( const cv::Mat *dsp_image_p, uByte_Type auBPortInNum )
{
  if( dsp_image_p != NULL)
  {
    if( auBPortInNum < IMLD_IN_PORT_MAX )
    {
      m_current_image.width = dsp_image_p->cols;

      if( auBPortInNum == IMLD_IN_PORT1 )
      {
        m_out1_image.width = dsp_image_p->cols;
      }
      else if( auBPortInNum == IMLD_IN_PORT2 )
      {
        m_out2_image.width = dsp_image_p->cols;
      }
      else
      {
        //IMLD_SET_FAILURE;
      }
    }
  }
}


/***************************************************************************************************
 *  Function Name   ：nvdSetImage_Height
 *  Description     ：This function sets the height data to CameraImage type data.
 *  Argument        ：height data of image
 *  Return          ：nothing special
 **************************************************************************************************/
void ImageLoader::nvdSetImage_Height( const cv::Mat *dsp_image_p, uByte_Type auBPortInNum )
{
  if( dsp_image_p != NULL)
  {
    if( auBPortInNum < IMLD_IN_PORT_MAX )
    {
      m_current_image.height = dsp_image_p->rows;

      if( auBPortInNum == IMLD_IN_PORT1 )
      {
        m_out1_image.height = dsp_image_p->rows;
      }
      else if( auBPortInNum == IMLD_IN_PORT2 )
      {
        m_out2_image.height = dsp_image_p->rows;
      }
      else
      {
        //IMLD_SET_FAILURE;
      }
    }
  }
}


/***************************************************************************************************
 *  Function Name   ：nvdSetImage_Bpp
 *  Description     ：This function sets the bpp data to CameraImage type data.
 *  Argument        ：bpp data(bit per pixel) of image
 *  Return          ：nothing special
 **************************************************************************************************/
void ImageLoader::nvdSetImage_Bpp( uHWord_Type auHW_bpp, uByte_Type auBPortInNum )
{
  if( auBPortInNum < IMLD_IN_PORT_MAX )
  {
    m_current_image.bpp = auHW_bpp;

    if( auBPortInNum == IMLD_IN_PORT1 )
    {
      m_out1_image.bpp = auHW_bpp;
    }
    else if( auBPortInNum == IMLD_IN_PORT2 )
    {
      m_out2_image.bpp = auHW_bpp;
    }
    else
    {
      //IMLD_SET_FAILURE;
    }
  }
}

/***************************************************************************************************
 *  Function Name   ：nvdSetImage_Length
 *  Description     ：This function sets the length data to CameraImage type data.
 *  Argument        ：width and height, bpp data of image.
 *  Return          ：nothing special
 **************************************************************************************************/
void ImageLoader::nvdSetImage_Length( RTC::CameraImage *m_current_image_p, uByte_Type auBPortInNum )
{
  if( m_current_image_p != NULL)
  {
    if( auBPortInNum < IMLD_IN_PORT_MAX )
    {
      m_current_image.pixels.length((m_current_image_p->width) * (m_current_image_p->height) * (m_current_image_p->bpp) / 8);

      if( auBPortInNum == IMLD_IN_PORT1 )
      {
        m_out1_image.pixels.length((m_current_image_p->width) * (m_current_image_p->height) * (m_current_image_p->bpp) / 8);
      }
      else if( auBPortInNum == IMLD_IN_PORT2 )
      {
        m_out2_image.pixels.length((m_current_image_p->width) * (m_current_image_p->height) * (m_current_image_p->bpp) / 8);
      }
      else
      {
        //IMLD_SET_FAILURE;
      }
    }
  }
}


/***************************************************************************************************
 *  Function Name   ：nuWSetImage_Pixels1
 *  Description     ：This function sets the pixels data to CameraImage type data.
 *  Argument        ：width and height, bpp data of image.
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type ImageLoader::nuWSetImage_Pixels1( cv::Mat dsp_image, RTC::CameraImage *m_current_image_p )
{
  uWord_Type auWReturn = IMLD_SET_FAILURE;

  if(m_current_image_p != NULL)
  {
    switch(m_current_image_p->bpp)
    {
      case IMLD_BPP_8:
      {
        for(int y = 0; y < m_current_image_p->height; y++)
        {
          memcpy((void *)&(m_out1_image.pixels[m_current_image_p->width * m_current_image_p->bpp / 8 * y]),
              dsp_image.ptr<unsigned char>(y), m_current_image_p->width * m_current_image_p->bpp / 8);
        }
        m_out1_imageOut.write();
        auWReturn = IMLD_SET_SUCCESS;
        break;
      }

      case IMLD_BPP_24:
      {
        for(int y = 0; y < m_current_image_p->height; y++)
        {
          memcpy((void *)&(m_out1_image.pixels[m_current_image_p->width * m_current_image_p->bpp / 8 * y]),
              dsp_image.ptr<unsigned char>(y), m_current_image_p->width * m_current_image_p->bpp / 8);
        }
        m_out1_imageOut.write();
        auWReturn = IMLD_SET_SUCCESS;
        break;
      }

      default:
#ifdef DEBUG_SW
        std::cerr << "Bits Par Pixels is not correct." << std::endl;
#endif
    }
  }

  return auWReturn;
}


/***************************************************************************************************
 *  Function Name   ：nuWSetImage_Pixels2
 *  Description     ：This function sets the pixels data to CameraImage type data.
 *  Argument        ：width and height, bpp data of image.
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type ImageLoader::nuWSetImage_Pixels2( cv::Mat dsp_image, RTC::CameraImage *m_current_image_p )
{
  uWord_Type auWReturn = IMLD_SET_FAILURE;

  if(m_current_image_p != NULL)
  {
    switch(m_current_image_p->bpp)
    {
      case IMLD_BPP_8:
      {
        for(int y = 0; y < m_current_image_p->height; y++)
        {
          memcpy((void *)&(m_out2_image.pixels[m_current_image_p->width * m_current_image_p->bpp / 8 * y]),
              dsp_image.ptr<unsigned char>(y), m_current_image_p->width * m_current_image_p->bpp / 8);
        }
        m_out2_imageOut.write();
        auWReturn = IMLD_SET_SUCCESS;
        break;
      }

      case IMLD_BPP_24:
      {
        for(int y = 0; y < m_current_image_p->height; y++)
        {
          memcpy((void *)&(m_out2_image.pixels[m_current_image_p->width * m_current_image_p->bpp / 8 * y]),
              dsp_image.ptr<unsigned char>(y), m_current_image_p->width * m_current_image_p->bpp / 8);
        }
        m_out2_imageOut.write();
        auWReturn = IMLD_SET_SUCCESS;
        break;
      }

      default:
#ifdef DEBUG_SW
        std::cerr << "Bits Par Pixels is not correct." << std::endl;
#endif
    }
  }

  return auWReturn;
}


/***************************************************************************************************
 *  Function Name   ：nuWGetFileSwConfig
 *  Description     ：This function returns the setup information of the configuration.
 *  Argument        ：nothing special
 *  Return          ：setup information of the configuration.
 **************************************************************************************************/
uWord_Type ImageLoader::nuWGetFileSwConfig( uByte_Type auBImldInPortNum )
{
  uWord_Type auWReturn = IMLD_FILE_LOAD_DISABLE;

  if( auBImldInPortNum < IMLD_IN_PORT_MAX )
  {
    if( auBImldInPortNum == IMLD_IN_PORT1 )
    {
       auWReturn = nuWGetFile1SwConfig();
    }
    else if( auBImldInPortNum == IMLD_IN_PORT2 )
    {
       auWReturn = nuWGetFile2SwConfig();
    }
    else
    {
      //FAILURE;
    }
  }

  return auWReturn;
}


/***************************************************************************************************
 *  Function Name   ：nuWJudgeNewPathIn
 *  Description     ：This function to confirmed the input of the new file path information.
 *  Argument        ：nothing special
 *  Return          ：valid or invalid of the input port.
 **************************************************************************************************/
uWord_Type ImageLoader::nuWJudgeNewPathIn( uByte_Type auBImldInPortNum )
{
  uWord_Type auWReturn = IMLD_PATH_IN_INVALID;

  if( auBImldInPortNum < IMLD_IN_PORT_MAX )
  {
    if( auBImldInPortNum == IMLD_IN_PORT1 )
    {
      if( m_in1_pathIn.isNew() == true )
      {
        m_in1_pathIn.read();
        m_in_path_fname = std::string(m_in1_path.data);
        if( nuWPathFileExists( m_in_path_fname ) == TRUE )
        {
          auWReturn = IMLD_PATH_IN_VALID;
        }
      }
    }
    else if( auBImldInPortNum == IMLD_IN_PORT2 )
    {
      if( m_in2_pathIn.isNew() == true )
      {
        m_in2_pathIn.read();
        m_in_path_fname = std::string(m_in2_path.data);
        if( nuWPathFileExists( m_in_path_fname ) == TRUE )
        {
          auWReturn = IMLD_PATH_IN_VALID;
        }
      }
    }
    else
    {
      //FAILURE;
    }
  }

  return auWReturn;
}


/***************************************************************************************************
 *  Function Name   ：nuWJudgeShadowPathIn
 *  Description     ：This function to confirmed the input of the new file path information.
 *  Argument        ：nothing special
 *  Return          ：valid or invalid of the input port.
 **************************************************************************************************/
uWord_Type ImageLoader::nuWJudgeShadowPathIn( uByte_Type auBImldInPortNum )
{
  uWord_Type auWReturn = IMLD_FILE_NAME_INVALID;

  if( auBImldInPortNum < IMLD_IN_PORT_MAX )
  {
    if( auBImldInPortNum == IMLD_IN_PORT1 )
    {
      if( m_stFilePath_shadow.validfname1 == IMLD_FILE_NAME_VALID )
      {
        m_stFilePath_shadow.validfname1 = IMLD_FILE_NAME_INVALID;
        auWReturn = IMLD_FILE_NAME_VALID;
      }
    }
    else if( auBImldInPortNum == IMLD_IN_PORT2 )
    {
      if( m_stFilePath_shadow.validfname2 == IMLD_FILE_NAME_VALID )
      {
        m_stFilePath_shadow.validfname2 = IMLD_FILE_NAME_INVALID;
        auWReturn = IMLD_FILE_NAME_VALID;
      }
    }
    else
    {
      //FAILURE;
    }
  }

  return auWReturn;
}


/***************************************************************************************************
 *  Function Name   ：nuWCompareFilePath
 *  Description     ：This function performs a comparison of the file path information.
 *  Argument        ：nothing special
 *  Return          ：The comparison result of the file path.
 **************************************************************************************************/
uWord_Type ImageLoader::nuWCompareFilePath( uByte_Type auBImldInPortNum )
{
  uWord_Type auWReturn = IMLD_FILE_PATH_MATCH;

  if( auBImldInPortNum < IMLD_IN_PORT_MAX )
  {
    if( auBImldInPortNum == IMLD_IN_PORT1 )
    {
      if( nuWPathFileExists( m_file1path_str ) == TRUE )
      {
        if( m_stFilePath.path_fname1.compare( m_file1path_str ) != 0 )
        {
          m_stFilePath.path_fname1 = m_file1path_str;
          auWReturn = IMLD_FILE_PATH_UNMATCH;
        }
      }
    }
    else if( auBImldInPortNum == IMLD_IN_PORT2 )
    {
      if( nuWPathFileExists( m_file2path_str ) == TRUE )
      {
        if( m_stFilePath.path_fname2.compare( m_file2path_str ) != 0 )
        {
          m_stFilePath.path_fname2 = m_file2path_str;
          auWReturn = IMLD_FILE_PATH_UNMATCH;
        }
      }
    }
    else
    {
      //FAILURE;
    }
  }

  return auWReturn;
}


/***************************************************************************************************
 *  Function Name   ：nvdSetFilePath
 *  Description     ：This function sets the file path information to the member variable.
 *  Argument        ：file path information
 *  Return          ：nothing special
 **************************************************************************************************/
void ImageLoader::nvdSetFilePath( std::string a_in_path_fname, uByte_Type auBImldInPortNum, uByte_Type auBFilePathOpt )
{
  if( auBImldInPortNum < IMLD_IN_PORT_MAX )
  {
    if( auBImldInPortNum == IMLD_IN_PORT1 )
    {
      m_stFilePath.path_fname1 = a_in_path_fname;
      m_stFilePath.validfname1 = IMLD_FILE_NAME_VALID;

      if( auBFilePathOpt ==  FILE_PATH_OPT_ON)
      {
        m_stFilePath_shadow.validfname1 = IMLD_FILE_NAME_INVALID;
      }
    }
    else if( auBImldInPortNum == IMLD_IN_PORT2 )
    {
      m_stFilePath.path_fname2 = a_in_path_fname;
      m_stFilePath.validfname2 = IMLD_FILE_NAME_VALID;

      if( auBFilePathOpt ==  FILE_PATH_OPT_ON)
      {
        m_stFilePath_shadow.validfname2 = IMLD_FILE_NAME_INVALID;
      }
    }
    else
    {
      //FAILURE;
    }
  }
}


/***************************************************************************************************
 *  Function Name   ：nvdSetFilePathShadow
 *  Description     ：This function sets the file path information to the member variable.
 *  Argument        ：file path information
 *  Return          ：nothing special
 **************************************************************************************************/
void ImageLoader::nvdSetFilePathShadow( std::string a_in_path_fname, uByte_Type auBImldInPortNum )
{
  if( auBImldInPortNum < IMLD_IN_PORT_MAX )
  {
    if( auBImldInPortNum == IMLD_IN_PORT1 )
    {
      m_stFilePath_shadow.path_fname1 = a_in_path_fname;
      m_stFilePath_shadow.validfname1 = IMLD_FILE_NAME_VALID;
    }
    else if( auBImldInPortNum == IMLD_IN_PORT2 )
    {
      m_stFilePath_shadow.path_fname2 = a_in_path_fname;
      m_stFilePath_shadow.validfname2 = IMLD_FILE_NAME_VALID;
    }
    else
    {
      //FAILURE;
    }
  }
}


/***************************************************************************************************
 *  Function Name   ：nuWGetFile1SwConfig
 *  Description     ：This function returns the setup information of the configuration for input 1.
 *  Argument        ：nothing special
 *  Return          ：setup information of the configuration for input 1.
 **************************************************************************************************/
uWord_Type ImageLoader::nuWGetFile1SwConfig( void )
{
  uWord_Type auWFileSwConfig = IMLD_FILE_LOAD_DISABLE;
  
  if( m_imld_file1sw < IMLD_FILE_LOAD_MAX )
  {
    auWFileSwConfig = static_cast<uWord_Type>(m_imld_file1sw);
  }

  return auWFileSwConfig;
}


/***************************************************************************************************
 *  Function Name   ：nuWGetFile1SwConfig
 *  Description     ：This function returns the setup information of the configuration for input 2.
 *  Argument        ：nothing special
 *  Return          ：setup information of the configuration for input 2.
 **************************************************************************************************/
uWord_Type ImageLoader::nuWGetFile2SwConfig( void )
{
  uWord_Type auWFileSwConfig = IMLD_FILE_LOAD_DISABLE;
  
  if( m_imld_file2sw < IMLD_FILE_LOAD_MAX )
  {
    auWFileSwConfig = static_cast<uWord_Type>(m_imld_file2sw);
  }

  return auWFileSwConfig;
}


/***************************************************************************************************
 *  Function Name   ：nuWPathFileExists
 *  Description     ：This function, the image file will confirm the existence.
 *  Argument        ：Path information of the image file
 *  Return          ：information of normal end or abnormal end.
 **************************************************************************************************/
uWord_Type ImageLoader::nuWPathFileExists( std::string apath_fname )
{
  uWord_Type auWResult = FALSE;
  LPCSTR file_exists;

  file_exists = apath_fname.c_str();
  if ( PathFileExists(file_exists) )
  {
    auWResult = TRUE;
  }
  else
  {
    std::cout << "Path of the directory or file is wrong." << std::endl;
  }

  return auWResult;
}


//--------------------------------------------------------------------------------------------------
extern "C"
{
 
  void ImageLoaderInit(RTC::Manager* manager)
  {
    coil::Properties profile(imageloader_spec);
    manager->registerFactory(profile,
                             RTC::Create<ImageLoader>,
                             RTC::Delete<ImageLoader>);
  }
  
};


